from abc import ABC
from typing import List

from credsweeper.common.constants import GroupType
from credsweeper.config.config import Config
from credsweeper.filters.filter import Filter
from credsweeper.filters.line_specific_key_check import LineSpecificKeyCheck
from credsweeper.filters.value_allowlist_check import ValueAllowlistCheck
from credsweeper.filters.value_array_dictionary_check import ValueArrayDictionaryCheck
from credsweeper.filters.value_blocklist_check import ValueBlocklistCheck
from credsweeper.filters.value_camel_case_check import ValueCamelCaseCheck
from credsweeper.filters.value_file_path_check import ValueFilePathCheck
from credsweeper.filters.value_hex_number_check import ValueHexNumberCheck
from credsweeper.filters.value_last_word_check import ValueLastWordCheck
from credsweeper.filters.value_method_check import ValueMethodCheck
from credsweeper.filters.value_not_allowed_pattern_check import ValueNotAllowedPatternCheck
from credsweeper.filters.value_pattern_check import ValuePatternCheck
from credsweeper.filters.value_similarity_check import ValueSimilarityCheck
from credsweeper.filters.value_string_type_check import ValueStringTypeCheck
from credsweeper.filters.value_token_check import ValueTokenCheck


class Group(ABC):
    """Abstract Group class"""

    def __init__(self, config: Config, rule_type: GroupType = GroupType.DEFAULT) -> None:
        """Config is required for filter group"""
        if rule_type == GroupType.KEYWORD:
            self.__filters = [  #
                ValueAllowlistCheck(),  #
                ValueArrayDictionaryCheck(),  #
                ValueBlocklistCheck(),  #
                ValueCamelCaseCheck(),  #
                ValueFilePathCheck(),  #
                ValueHexNumberCheck(),  #
                ValueLastWordCheck(),  #
                ValueMethodCheck(),  #
                ValueSimilarityCheck(),  #
                ValueStringTypeCheck(check_for_literals=config.check_for_literals),  #
                ValueTokenCheck(),  #
            ]
            if not config.doc:
                self.__filters.extend([ValuePatternCheck(), ValueNotAllowedPatternCheck()])
        elif rule_type == GroupType.PATTERN:
            self.__filters = [  #
                LineSpecificKeyCheck(),  #
                ValuePatternCheck(),  #
            ]
        else:
            # GroupType.DEFAULT
            self.__filters = []

    @property
    def filters(self) -> List[Filter]:
        """property getter"""
        return self.__filters

    @filters.setter
    def filters(self, filters: List[Filter]) -> None:
        """property setter"""
        self.__filters = filters



import contextlib
import logging
import re
from abc import ABC
from typing import List, Optional

from credsweeper.common.constants import MAX_LINE_LENGTH
from credsweeper.credentials.candidate import Candidate
from credsweeper.deep_scanner.abstract_scanner import AbstractScanner
from credsweeper.file_handler.data_content_provider import DataContentProvider
from credsweeper.utils.util import Util

logger = logging.getLogger(__name__)

# 8 bytes are encoded to 12 symbols 12345678 -> MTIzNDU2Nzg=
MIN_ENCODED_DATA_LEN = 12


class EncoderScanner(AbstractScanner, ABC):
    """Implements recursive iteration when data might be encoded from base64"""

    BASE64_PATTERN = re.compile(
        rb"(\xFF\xFE|\xFE\xFF)?("
        rb"(?:(?P<a>[A-Z])|(?P<b>[a-z])|(?P<c>[0-9/+])|[\s\x00\\])+(?(a)(?(b)(?(c)(=+|$)|(?!x)x)|(?!x)x)|(?!x)x)|"
        rb"(?:(?P<e>[A-Z])|(?P<f>[a-z])|(?P<g>[0-9_-])|[\s\x00\\])+(?(e)(?(f)(?(g)(=+|$)|(?!x)x)|(?!x)x)|(?!x)x))")

    @staticmethod
    def match(data: bytes) -> bool:
        """Check if data MAY be base64 encoded with whitespaces (escaping too)"""
        if len(data) >= MIN_ENCODED_DATA_LEN \
                and EncoderScanner.BASE64_PATTERN.match(data, pos=0, endpos=MAX_LINE_LENGTH):
            return True
        return False

    @staticmethod
    def decode(text: str) -> Optional[bytes]:
        """Decodes base64 text with cleaning whitespaces. Returns None when the decoding fails"""
        with contextlib.suppress(Exception):
            return Util.decode_base64(text=Util.PEM_CLEANING_PATTERN.sub(r'', text).replace('\\', ''),
                                      padding_safe=True,
                                      urlsafe_detect=True)
        return None

    def data_scan(
            self,  #
            data_provider: DataContentProvider,  #
            depth: int,  #
            recursive_limit_size: int) -> Optional[List[Candidate]]:
        """Tries to decode data from base64 encode to bytes and scan as bytes again"""
        if decoded := EncoderScanner.decode(data_provider.text):
            decoded_data_provider = DataContentProvider(data=decoded,
                                                        file_path=data_provider.file_path,
                                                        file_type=data_provider.file_type,
                                                        info=f"{data_provider.info}|BASE64")
            new_limit = recursive_limit_size - len(decoded_data_provider.data)
            return self.recursive_scan(decoded_data_provider, depth, new_limit)
        return None



import re
from typing import Optional

from credsweeper.config.config import Config
from credsweeper.credentials.line_data import LineData
from credsweeper.file_handler.analysis_target import AnalysisTarget
from credsweeper.filters.filter import Filter


class ValueHexNumberCheck(Filter):
    """Check value if it is a value up to 64 bits hex representation"""

    HEX_08_64_VALUE_REGEX = re.compile(r"^0x[0-9a-f]{1,16}$")

    def __init__(self, config: Optional[Config] = None) -> None:
        pass

    def run(self, line_data: LineData, target: AnalysisTarget) -> bool:
        """Run filter checks on received credential candidate data 'line_data'.

        Args:
            line_data: credential candidate data
            target: multiline target from which line data was obtained

        Return:
            True, if need to filter candidate and False if left

        """
        value = line_data.value.lower()
        if ValueHexNumberCheck.HEX_08_64_VALUE_REGEX.match(value):
            return True
        return False



import logging
from abc import ABC
from typing import List, Optional, Union

import jks

from credsweeper.common.constants import Severity, Confidence
from credsweeper.credentials.candidate import Candidate
from credsweeper.deep_scanner.abstract_scanner import AbstractScanner
from credsweeper.file_handler.data_content_provider import DataContentProvider

logger = logging.getLogger(__name__)


class JksScanner(AbstractScanner, ABC):
    """Implements jks scanning"""

    @staticmethod
    def match(data: Union[bytes, bytearray]) -> bool:
        """According https://en.wikipedia.org/wiki/List_of_file_signatures - jks"""
        if data.startswith(b"\xFE\xED\xFE\xED"):
            return True
        return False

    def data_scan(
            self,  #
            data_provider: DataContentProvider,  #
            depth: int,  #
            recursive_limit_size: int) -> Optional[List[Candidate]]:
        """Tries to scan JKS to open with standard password"""
        for pw_probe in self.config.bruteforce_list:
            value = repr(pw_probe)
            try:
                keystore = jks.KeyStore.loads(data_provider.data, pw_probe, try_decrypt_keys=True)
                # the password probe has passed, it will be the value
                if keystore.private_keys or keystore.secret_keys:
                    severity = Severity.HIGH
                    confidence = Confidence.STRONG
                    info = f"{data_provider.info}|JKS:default password"
                    rule_name = f"JKS private key with password {value}"
                else:
                    severity = Severity.LOW
                    confidence = Confidence.WEAK
                    info = f"{data_provider.info}|JKS:sensitive data"
                    rule_name = f"JKS sensitive data with password {value}"
                candidate = Candidate.get_dummy_candidate(
                    self.config,  #
                    data_provider.file_path,  #
                    data_provider.file_type,  #
                    info,  #
                    rule_name)
                candidate.severity = severity
                candidate.confidence = confidence
                candidate.line_data_list[0].line = candidate.line_data_list[0].value = value
                candidate.line_data_list[0].value_start = 0
                candidate.line_data_list[0].value_end = len(value)
                return [candidate]
            except Exception as jks_exc:
                logger.debug(f"{data_provider.file_path}:{pw_probe}:{jks_exc}")
        return None
